#include <iostream>
#include <fstream>
#include "freeglut.h"
using namespace std;

#include "ModelLoader.h" 

// Load a "obj"-type model
ModelLoader::ModelLoader(char *filename) {
	const int MAX_LEN = 255;

	m_hasNormal = false;
	m_hasTexture = false;

	char *line = new char[MAX_LEN];
	char *line_back = new char[MAX_LEN];
	char wspace[] = " \t\n";
	char separator[] = "/";
	char *token;
	int indices[3];
	double x, y, z;
	double tex_u, tex_v;

	ifstream file(filename);
	if ( !file ) {
		cerr << "ERR: cannot open file: " << filename << " exiting." << endl;
		exit ( -1 );
	}

	while ( !file.eof() ) {
		file.getline( line, MAX_LEN - 1);
		if ( line[0] == '#' )
			continue;
		else if ( !strcmp( line, "" ) )
			continue;
		else {
			strcpy( line_back, line );       
			token = strtok( line, wspace);
			if ( !strcmp( token, "v" ) ) {
				x = atof( strtok( NULL, wspace ) );
				y = atof( strtok( NULL, wspace ) );
				z = atof( strtok( NULL, wspace ) );
				m_vertices.push_back(Vertex( x, y, z ));
			}
			else if ( !strcmp( token, "vn" ) ) {
				x = atof( strtok( NULL, wspace ) );
				y = atof( strtok( NULL, wspace ) );
				z = atof( strtok( NULL, wspace ) );
				m_normals.push_back(Normal( x, y, z ));
			}
			else if ( !strcmp( token, "vt" ) ) {
				tex_u = atof( strtok( NULL, wspace) );
				tex_v = atof( strtok( NULL, wspace) );
				m_tex.push_back(TexCoord(tex_u, tex_v));
			}
			else if ( !strcmp( token, "f" ) ) {
				m_faces.push_back(Triangle());
				Triangle& curFace = m_faces.back();

				int nVertex = 0;
				for (char *p = strtok( NULL, wspace ); p; p = strtok( NULL, wspace ) ) {
					indices[0] = -1; 
					indices[1] = -1;
					indices[2] = -1;
					char* pos = p;
					int len = (int)strlen(p);
					for ( int j=0, i=0;  j <= len && i < 3; j++ ) {
						if ( p[j] == '/' || p[j] == 0) {
							p[j] = 0;
							indices[i++] = atoi( pos );
							pos = p + j+1;
						}
					}
					curFace.m_idxV[nVertex] = indices[0];
					curFace.m_idxT[nVertex] = indices[1];
					curFace.m_idxN[nVertex] = indices[2];
					nVertex++;
					if (indices[1] > 0)		// this obj file has texture
						m_hasTexture = true;
					if( indices[2] > 0 )	// this obj file has vertex normal
						m_hasNormal = true;
				}
				if (nVertex != 3) {
					cerr << "ERR: only support for triangles for now" << endl;
					exit(-1);
				}
			}			
			else if ( !strcmp( token, "" ) ) {       // blank line
			}
			else {
				cout <<line_back <<endl;
			}
		}
	}

	// 
	if (!m_hasNormal) {
		for (int i = 0; i < m_faces.size(); ++i) {
			Triangle& curFace = m_faces[i];
			curFace.calcNormal(m_vertices[curFace.m_idxV[0]], m_vertices[curFace.m_idxV[1]], m_vertices[curFace.m_idxV[2]]);
		}
	}
}

// Draw a model 
void ModelLoader::Draw() {
	int i;      
	for (int f = 0; f < m_faces.size(); f++) {
		Triangle& curFace = m_faces[f];        
		glBegin(GL_TRIANGLES);
		for (int v = 0; v < 3; v++) {
			if (!m_hasNormal) {
				if (v == 0) // other vertices will have same normal
					glNormal3d( curFace.m_faceNormal.m_nx, curFace.m_faceNormal.m_ny, curFace.m_faceNormal.m_nz);				
			}
			else {
				i = curFace.m_idxN[v] - 1; // assumption: start index = 1 rather than 0
				glNormal3d(m_normals[i].m_nx, m_normals[i].m_ny, m_normals[i].m_nz);
			}
			if (m_hasTexture) {
				i = curFace.m_idxT[v] - 1; // assumption: start index = 1 rather than 0
				glTexCoord2d(m_tex[i].m_u, m_tex[i].m_v);
			}

			i = curFace.m_idxV[v] - 1; // assumption: start index = 1 rather than 0
			glVertex3d(m_vertices[i].m_x, m_vertices[i].m_y, m_vertices[i].m_z);			
		}
		glEnd();
	}
}
