#pragma once

#include <vector>
#include <stdlib.h>
#include <math.h>
using std::vector;

// definition of vertex  
class Vertex {
public:
	double m_x, m_y, m_z;

	Vertex(double x, double y, double z) {
		m_x = x, m_y = y, m_z = z;
	}
	Vertex() {
		m_x = m_y = m_z = 0.0;
	}
};

// definition of normal (used for shading)
class Normal {
public:
	double m_nx, m_ny, m_nz;
	Normal(double nx, double ny, double nz) {
		m_nx = nx, m_ny = ny, m_nz = nz;
	}
	Normal() {
		m_nx = m_nx = m_nz = 0.0;
	}
};

// definition of texture coordinate (used for texture mapping)
class TexCoord {
public:
	double m_u, m_v;
	TexCoord(double u, double v) {
		m_u = u, m_v = v;
	}
};

// definition of triangle
class Triangle {
public:		
	Normal m_faceNormal;
	int m_idxV[3];
	int m_idxN[3];
	int m_idxT[3];

	Triangle() {}
	void calcNormal(Vertex& v0, Vertex& v1, Vertex& v2) {
		double ax, ay, az;
		double bx, by, bz;
		double nx, ny, nz;
		ax = v1.m_x - v0.m_x;
		ay = v1.m_y - v0.m_y;
		az = v1.m_z - v0.m_z;
		bx = v2.m_x - v0.m_x;
		by = v2.m_y - v0.m_y;
		bz = v2.m_z - v0.m_z;
		nx = ay*bz - az*by;
		ny = az*bx - ax*bz;
		nz = ax*by - ay*bx;
		m_faceNormal = Normal(nx, ny, nz);
	}
};

// Define a model
class ModelLoader  {
public:
	vector<Vertex> m_vertices;
	vector<TexCoord> m_tex;
	vector<Normal> m_normals;
	vector<Triangle> m_faces;

	bool m_hasNormal;
	bool m_hasTexture;

	ModelLoader(char *filename);
	void Draw();
};
