#include <iostream>
#include <stdlib.h>
#include <math.h>

#include "freeglut.h"
#include <GL/glu.h>
#include "Matrix.h"
#include "ModelLoader.h"

#define M_PI 3.1415926535897932384626433832795

int g_frame = 0;		// frame count
int g_width, g_height;	// window size

// Define some variance for the teapot
MyMatrix g_teapot2world;
ModelLoader* g_teapot;
int g_teapotID;

void setCamera() {
	// TODO... we might want to change eye position and directions 
}

// draw the teapot model (obj)
void drawTeapot() {
	if (g_frame == 0)	{
		// Initialization for the first frame.

		// Read information from teapot.obj (you should specify your model path in your system).
		g_teapot = new ModelLoader("../model/teapot.obj");

		// Create display list, and you can draw an object using its ID.
		g_teapotID = glGenLists(1);				// Create display lists
		glNewList(g_teapotID, GL_COMPILE);		// Begin a display list
		g_teapot->Draw();						// Draw the teapot on display list.
		glEndList();							// End of your list

		glPushMatrix();						// Push the current matrix of GL into stack.
		glLoadIdentity();					// Set the GL matrix Identity matrix.		

		glTranslated(0, -0.5, -10.0);
		
		glGetDoublev(GL_MODELVIEW_MATRIX, g_teapot2world.matrix());	// Read the modelview matrix about location and direction set above, and store it to g_teapot2world matrix.
		glPopMatrix();	// Pop the matrix on stack to GL.
	}

	glPushMatrix();		// Push the current matrix of GL into stack. 

	// The information about location of an object to be drawn is stored in teapot2wld matrix. If you change the value of the g_teapot2world matrix or the current matrix, teapot can be transformed.	
	glMultMatrixd(g_teapot2world.matrix());

	// Set a default material (color) for shading the teapot
	float frontColor[] = { 0.2, 0.8, 0.1, 1.0 };
	glEnable(GL_LIGHTING);
	glMaterialfv(GL_FRONT, GL_AMBIENT, frontColor);		// Set ambient color.
	glMaterialfv(GL_FRONT, GL_DIFFUSE, frontColor);		// Set diffuse color.

	glCallList(g_teapotID);		// Draw an object from predefined display lists. 
	glPopMatrix();				// Pop the matrix in stack to GL. 
}


void display() {					
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);			// Clear the screen and depth buffer

	setCamera();												// Setup Camera's position and direction

	// Draw your objects here
	drawTeapot();		// Draw a teapot

	glFlush();			// Flush the graphics pipeline
	g_frame += 1;		// Increment the frame count
}

void reshape(int w, int h) {
	g_width = w;
	g_height = h;

	glViewport(0, 0, g_width, g_height);	// Define your viewport

	glMatrixMode(GL_PROJECTION);            // Select The Projection Matrix
	glLoadIdentity();                       // Reset The Projection Matrix

	// Define the perspective viewing volume
	double aspect = g_width / double(g_height);
	gluPerspective(45, aspect, 1, 1000);

	glMatrixMode(GL_MODELVIEW);             // Select The Modelview Matrix
	glLoadIdentity();                       // Reset The Projection Matrix
}

void initOpenGL() {	
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);	// Setup background color
	glClearDepth(1.0f);					    // Setup depth buffer
	glDepthFunc(GL_LEQUAL);				    // Setup how to test depth (less or equal)
	glEnable(GL_DEPTH_TEST);			    // Enable depth test	
	glShadeModel(GL_FLAT);		            // Setup shading mode								   
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); // Set perspective calculations to be more accurate

	// Initialize the matrix stacks
	reshape(g_width, g_height);

	// Define lights in the scene
	float lightDirection[] = { 1.0, 1.0, 1.0, 1.0 };
	float ambientIntensity[] = { 0.1, 0.1, 0.1, 1.0 };
	float lightIntensity[] = { 0.5, 0.5, 0.5, 1.0 };
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambientIntensity);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightIntensity);
	glLightfv(GL_LIGHT0, GL_POSITION, lightDirection);
	glEnable(GL_LIGHT0);
}

void mouseButton(int button, int state, int x, int y) {
	// TODO... we need to handle mouse button events here

	glutPostRedisplay();
}

void mouseDrag(int x, int y) {
	// TODO... we need to handle mouse drag events here

	glutPostRedisplay();
}

void keyboard(unsigned char key, int x, int y) {
	// TODO... we need to handle keyboard events here

	glutPostRedisplay();
}

//------------------------------------------------------------------------------
void main(int argc, char* argv[]) {
	g_width = 800;
	g_height = 600;
	g_frame = 0;

	glutInit(&argc, argv);						// Initialize openGL.
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);// Initialize a display mode (e.g., double buffer and RGB color).
	glutInitWindowSize(g_width, g_height);		// Choose your window size
	glutInitWindowPosition(100, 100);			// Determine the poisition of your window
	glutCreateWindow("Your Student ID here");	// Create an window with a specified name
	glutDisplayFunc(display);					// Register the main display function where you draw models
	glutReshapeFunc(reshape);					// Register reshape function to be called when your window is resized
	glutKeyboardFunc(keyboard);					// Register a handler for keyboard events
	glutMouseFunc(mouseButton);					// Register a handler for mouse button events
	glutMotionFunc(mouseDrag);					// Register a handler for mouse drag events
	
	initOpenGL();								// Initialize for OpenGL stuffs.
	glutMainLoop();								// Start the main GL loop (ready to handle all events)
}

